/*
 * Copyright (c) 1999 Greg Haerr <greg@censoft.com>
 *
 * 2bpp Packed Linear Video Driver for MicroWindows
 * 	This driver is written for the Vr41xx Palm PC machines
 * 	The screen is 320x240x4
 *
 * 	In this driver, LINELEN is line byte length, not line pixel length
 */
/*#define NDEBUG*/
#include <assert.h>
#include <string.h>
#include "../device.h"
#include "fb.h"

static unsigned char notmask[4] = { 0x3f, 0xcf, 0xf3, 0xfc};

/* Set pixel at x, y, to pixelval c*/
void
linear2_drawpixel(ADDR8 addr, COORD x, COORD y, PIXELVAL c)
{
	assert (x >= 0 && x < XRES);
	assert (y >= 0 && y < YRES);
	assert (c >= 0 && c < NCOLORS);

	++drawing;
	addr += x/4 + y * LINELEN;
	if(gr_mode == MODE_XOR)
		*addr = (*addr & notmask[x&3]) ^ (c << ((3-(x&3))*2));
	else
		*addr = (*addr & notmask[x&3]) | (c << ((3-(x&3))*2));
	--drawing;
}

/* Read pixel at x, y*/
PIXELVAL
linear2_readpixel(ADDR8 addr, COORD x, COORD y)
{
	assert (x >= 0 && x < XRES);
	assert (y >= 0 && y < YRES);

	return (addr[x/4 + y * LINELEN] >> ((3-(x&3))*2) ) & 0x03;
}

/* Draw horizontal line from x1,y to x2,y not including final point*/
void
linear2_drawhorzline(ADDR8 addr, COORD x1, COORD x2, COORD y, PIXELVAL c)
{
	assert (x1 >= 0 && x1 < XRES);
	assert (x2 >= 0 && x2 <= XRES);
	assert (x2 >= x1);
	assert (y >= 0 && y < YRES);
	assert (c >= 0 && c < NCOLORS);

	++drawing;
	addr += x1/4 + y * LINELEN;
	if(gr_mode == MODE_XOR) {
		while(x1 < x2) {
			*addr = (*addr & notmask[x1&3]) ^ (c << ((3-(x1&3))*2));
			if((++x1 & 3) == 0)
				++addr;
		}
	} else {
		while(x1 < x2) {
			*addr = (*addr & notmask[x1&3]) | (c << ((3-(x1&3))*2));
			if((++x1 & 3) == 0)
				++addr;
		}
	}
	--drawing;
}

/* Draw a vertical line from x,y1 to x,y2 not including final point*/
void
linear2_drawvertline(ADDR8 addr, COORD x, COORD y1, COORD y2, PIXELVAL c)
{
	assert (x >= 0 && x < XRES);
	assert (y1 >= 0 && y1 < YRES);
	assert (y2 >= 0 && y2 <= YRES);
	assert (y2 >= y1);
	assert (c >= 0 && c < NCOLORS);

	++drawing;
	addr += x/4 + y1 * LINELEN;
	if(gr_mode == MODE_XOR)
		while(y1++ < y2) {
			*addr = (*addr & notmask[x&3]) ^ (c << ((3-(x&3))*2));
			addr += LINELEN;
		}
	else
		while(y1++ < y2) {
			*addr = (*addr & notmask[x&3]) | (c << ((3-(x&3))*2));
			addr += LINELEN;
		}
	--drawing;
}

void
linear2_blit(ADDR8 dstaddr, COORD destx, COORD desty, COORD w, COORD h,
	ADDR8 srcaddr, COORD srcx, COORD srcy)
{
	ADDR8	dst;
	ADDR8	src;
	int	i;

	assert (destx >= 0 && destx < XRES);
	assert (desty >= 0 && desty < YRES);
	assert (w > 0);
	assert (h > 0);
	assert (srcx >= 0 && srcx <= XRES);
	assert (srcy >= 0 && srcy < YRES);

	++drawing;
	// FIXME: LINELEN will change on offscreen bitmaps
	while(--h >= 0) {
		dst = dstaddr + destx/4 + desty * LINELEN;
		src = srcaddr + srcx/4 + srcy * LINELEN;
		for(i=0; i<w; ++i) {
			COORD	dx = destx;
			COORD	sx = srcx;
			*dst = (*dst & notmask[dx&3]) |
			   ((*src >> ((3-(sx&3))*2) & 0x03) << ((3-(dx&3))*2));
			if((++dx & 3) == 0)
				++dst;
			if((++sx & 3) == 0)
				++src;
		}
	}
	--drawing;
}

/* Return framebuffer mmap size*/
int
linear2_init(void)
{
	return YRES * LINELEN * sizeof(unsigned char);
}

FBENTRY fblinear2 = {
	(void *)linear2_init,
	(void *)linear2_drawpixel,
	(void *)linear2_readpixel,
	(void *)linear2_drawhorzline,
	(void *)linear2_drawvertline,
	(void *)linear2_blit
};
